@@ -57,8 +57,12 @@ func TestText(t *testing.T) {
 	}{
 		{"", "", ErrBadText},
 		{"=", "=", nil},
-		{"= cap_chown+iep cap_chown-i", "= cap_chown+ep", nil},
-		{"= cap_setfcap,cap_chown+iep cap_chown-i", "= cap_setfcap+eip cap_chown+ep", nil},
+		{"= cap_chown+iep cap_chown-i", "cap_chown=ep", nil},
+		{"= cap_setfcap,cap_chown+iep cap_chown-i", "cap_setfcap=eip cap_chown+ep", nil},
+		{"cap_setfcap,cap_chown=iep cap_chown-i", "cap_setfcap=eip cap_chown+ep", nil},
+		{"=i =p", "=p", nil},
+		{"all+pie", "=eip", nil},
+		{"all=p+ie-e", "=ip", nil},
 	}
 	for i, v := range vs {
 		c, err := FromText(v.from)
@@ -208,3 +212,42 @@ func TestIAB(t *testing.T) {
 		}
 	}
 }
+
+func TestFuncLaunch(t *testing.T) {
+	if _, err := FuncLauncher(func(data interface{}) error {
+		return nil
+	}).Launch(nil); err != nil {
+		t.Fatalf("trivial launcher failed: %v", err)
+	}
+
+	before, err := singlesc.prctlrcall(prGetKeepCaps, 0, 0)
+	if err != nil {
+		t.Fatalf("failed to get PR_KEEP_CAPS: %v", err)
+	}
+
+	if _, err := FuncLauncher(func(data interface{}) error {
+		was, ok := data.(int)
+		if !ok {
+			return fmt.Errorf("data was not an int: %v", data)
+		}
+		if _, err := Prctlw(prSetKeepCaps, uintptr(1-was)); err != nil {
+			return err
+		}
+		if v, err := Prctl(prGetKeepCaps); err != nil {
+			return err
+		} else if v == was {
+			return fmt.Errorf("PR_KEEP_CAPS unchanged: got=%d, want=%v", v, 1-was)
+		}
+		// All good.
+		return nil
+	}).Launch(before); err != nil {
+		t.Fatalf("trivial launcher failed: %v", err)
+	}
+
+	// Now validate that the main process is still OK.
+	if after, err := singlesc.prctlrcall(prGetKeepCaps, 0, 0); err != nil {
+		t.Fatalf("failed to get PR_KEEP_CAPS: %v", err)
+	} else if before != after {
+		t.Fatalf("FuncLauncher leaked privileged state: got=%v want=%v", after, before)
+	}
+}
