@@ -1,4 +1,3 @@
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,7 +24,6 @@ struct test_case_s {
     const char **envp;
     const char *iab;
     cap_mode_t mode;
-    int launch_abort;
     int result;
     int (*callback_fn)(void *detail);
 };
@@ -41,9 +39,7 @@ struct test_case_s {
 static int clean_out(void *data) {
     cap_t empty;
     empty = cap_init();
-    if (cap_set_proc(empty) != 0) {
-	_exit(1);
-    }
+    cap_set_proc(empty);
     cap_free(empty);
     return 0;
 }
@@ -67,10 +63,6 @@ int main(int argc, char **argv) {
 	    .args = { "../progs/tcapsh-static", "--is-uid=123" },
 	    .result = 256
 	},
-	{
-	    .args = { "/", "won't", "work" },
-	    .launch_abort = 1,
-	},
 	{
 	    .args = { "../progs/tcapsh-static", "--is-uid=123" },
 	    .uid = 123,
@@ -80,7 +72,7 @@ int main(int argc, char **argv) {
 	    .args = { "../progs/tcapsh-static", "--is-uid=123" },
 	    .callback_fn = &clean_out,
 	    .uid = 123,
-	    .launch_abort = 1,
+	    .result = 256
 	},
 	{
 	    .args = { "../progs/tcapsh-static", "--is-gid=123" },
@@ -120,33 +112,16 @@ int main(int argc, char **argv) {
 	},
     };
 
-    if (errno != 0) {
-	perror("unexpected initial value for errno");
-	exit(1);
-    }
-
     cap_t orig = cap_get_proc();
-    if (orig == NULL) {
-	perror("failed to get process capabilities");
-	exit(1);
-    }
 
     int success = 1, i;
     for (i=0; vs[i].pass_on != NO_MORE; i++) {
-	cap_launch_t attr = NULL;
 	const struct test_case_s *v = &vs[i];
-	if (cap_launch(attr, NULL) != -1) {
-	    perror("NULL launch didn't fail");
-	    exit(1);
-	}
 	printf("[%d] test should %s\n", i,
-	       v->result || v->launch_abort ? "generate error" : "work");
-	if (v->args[0] != NULL) {
+	       v->result ? "generate error" : "work");
+	cap_launch_t attr;
+	if (v->args != NULL) {
 	    attr = cap_new_launcher(v->args[0], v->args, v->envp);
-	    if (attr == NULL) {
-		perror("failed to obtain launcher");
-		exit(1);
-	    }
 	    if (v->callback_fn != NULL) {
 		cap_launcher_callback(attr, v->callback_fn);
 	    }
@@ -185,40 +160,34 @@ int main(int argc, char **argv) {
 	pid_t child = cap_launch(attr, NULL);
 
 	if (child <= 0) {
-	    fprintf(stderr, "[%d] failed to launch: ", i);
-	    perror("");
-	    if (!v->launch_abort) {
-		success = 0;
-	    }
+	    fprintf(stderr, "[%d] failed to launch", i);
+	    perror(":");
+	    success = 0;
 	    continue;
 	}
 	if (cap_free(attr)) {
-	    fprintf(stderr, "[%d] failed to free launcher: ", i);
-	    perror("");
+	    fprintf(stderr, "[%d] failed to free launcher", i);
+	    perror(":");
 	    success = 0;
 	}
 	int result;
 	int ret = waitpid(child, &result, 0);
 	if (ret != child) {
-	    fprintf(stderr, "[%d] failed to wait: ", i);
-	    perror("");
+	    fprintf(stderr, "[%d] failed to wait", i);
+	    perror(":");
 	    success = 0;
 	    continue;
 	}
 	if (result != v->result) {
-	    fprintf(stderr, "[%d] bad result: got=%d want=%d: ", i, result,
+	    fprintf(stderr, "[%d] bad result: got=%d want=%d", i, result,
 		    v->result);
-	    perror("");
+	    perror(":");
 	    success = 0;
 	    continue;
 	}
     }
 
     cap_t final = cap_get_proc();
-    if (final == NULL) {
-	perror("unable to get final capabilities");
-	exit(1);
-    }
     if (cap_compare(orig, final)) {
 	char *was = cap_to_text(orig, NULL);
 	char *is = cap_to_text(final, NULL);
@@ -230,10 +199,10 @@ int main(int argc, char **argv) {
     cap_free(final);
     cap_free(orig);
 
-    if (!success) {
+    if (success) {
+	printf("cap_launch_test: PASSED\n");
+    } else {
 	printf("cap_launch_test: FAILED\n");
 	exit(1);
     }
-    printf("cap_launch_test: PASSED\n");
-    exit(0);
 }
